/* ------------------------------------------------------------------------ 
 MIRS0202 PWM-Encoder Module Control Methods Object

                     From ATLMIRS Documents
                     Written by Hajime Wakabayashi
                     ver 0.1alpha Jan. 31 2004
   ------------------------------------------------------------------------ */

/*
 --- Refference ---

 void PWM::Start(int Period), void PWM::Stop(),void PWM::Restart() :
    Set thread status in PWM-Encoder Module.

 struct CountData PWM::Get() :
    Get Count/Sum Value from Motor.
    See also "pwm.h"

 void PWM::Set(Mode,[arg1,arg2]) :
    Set Mode of Motor
    Mode:
      MODE_KEEP
      MODE_DUTY
      MODE_STOP
      MODE_RETURN
      MODE_LR
      MODE_STR
      MODE_HANKEI
*/

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "pwm.h"

/* Construction/Destruction ----------------------------------------------- */
PWM::PWM(void) {
	cRb = 0; cLb = 0;
}

PWM::~PWM(void) {

}
/* ------------------------------------------------------------------------ */
/* Main Methods */
/* ------------------------------------------------------------------------ */
void PWM::Start(int Period) {
	PWM::FSetStatus(START_TASK, Period);
	usleep(Period);
	PWM::Set(MODE_DUTY,128,128);
}

/* ------------------------------------------------------------------------ */
void PWM::Restart(void) {
	PWM::FSetStatus(RESTART_TASK);
	usleep(50000);
	PWM::Set(MODE_DUTY,128,128);
}

/* ------------------------------------------------------------------------ */
void PWM::Stop(void) {
	PWM::Set(MODE_DUTY,128,128);
	usleep(50000);
	PWM::FSetStatus(STOP_TASK);
}

/* ------------------------------------------------------------------------ */
void PWM::Tune(int L, int R) {
	PWM::FSetStatus(TUNE_SET,L*100,R*100);
}

/* ------------------------------------------------------------------------ */
void PWM::FSetStatus(int Mode, ...) {
	fd_set wfds;
	struct timeval tv;
	va_list marker;
	struct Message msg;
	
	/* Open RT-FIFO */
	if((fd5 = open("/dev/rtf5", O_WRONLY)) < 0){
		fprintf(stderr, "Cannot Open RT-FIFO /dev/rtf5\n");
		exit(1);
	}

	msg.Task=0;
	msg.Command=0;
	msg.Period=0;

	/* Set Priod if Mode Equals START_TASK */
	if (Mode == START_TASK) {
		va_start( marker, Mode );
		msg.Period = va_arg(marker,int);
		va_end( marker );
	} else if (Mode == TUNE_SET) {
		va_start( marker, Mode );
		va_end( marker );
	}
	/* Set Command */
	msg.Command = Mode;
	msg.Task = 2;

	FD_ZERO(&wfds);
	FD_SET(fd5, &wfds);
	tv.tv_sec = 0;
	/* 100ms */
	tv.tv_usec = 100000;

	if(select(FD_SETSIZE, NULL, &wfds, NULL, &tv) > 0){
		write(fd5, &msg, sizeof(msg));
	} else {
		fprintf(stderr, "Can't send a command to RT-task\n");
		exit(1);
	}
	/* Close Handle */
	close(fd5); 
}


/* ------------------------------------------------------------------------ */
void PWM::Set(int Mode,...) {
	va_list marker;
	int Lv,Rv;
	
	va_start( marker, Mode );

	switch(Mode) {
		case MODE_DUTY:
			Lv = va_arg(marker,int);
			Rv = va_arg(marker,int);
			FSet(Lv,Rv,0,MODE_DUTY);
			break;
		case MODE_STOP:
			Lv = va_arg(marker,int);
			Rv = va_arg(marker,int);
			FSet(Lv,Rv,0,MODE_STOP);
			break;
		case MODE_RETURN:
			break;
		case MODE_LR:
			Lv = va_arg(marker,int);
			Rv = va_arg(marker,int);
			FSet(Lv,Rv,0,MODE_LR);
			break;
		case MODE_STR:
			Lv = va_arg(marker,int);
			Rv = va_arg(marker,int);
			FSet(Lv,Rv,0,MODE_STR);
			break;
		case MODE_HANKEI:
			Lv = va_arg(marker,int);
			Rv = va_arg(marker,int);
			FSet(Lv,Rv,0,MODE_HANKEI);
			break;
		default:
			break;
	}

	va_end( marker );
}
/* ------------------------------------------------------------------------ */
void PWM::FSet(int L, int R, int T, int M) {
	struct SpeedData pwm;

	usleep(10000);
	
	/* Open RT-FIFO */
	if ((fd4 = open("/dev/rtf4", O_WRONLY)) < 0) {
		fprintf(stderr, "Cannot Open RT-FIFO /dev/rtf4\n");
		exit(1);
	}
	
	/* Set SpeedData */
	pwm.LSpeed = L;
	pwm.RSpeed = R;
	pwm.Tup = T;
	pwm.Mode = M;
	
	/* Write Data */
	if (write(fd4, &pwm, sizeof(pwm)) < 0){
		fprintf(stderr, "Cannot Send Data to RT-FIFO /dev/rtf4\n");
		exit(1);
	}
	/* Close Handle */
	close(fd4);
}

/* ------------------------------------------------------------------------ */
CountData PWM::Get(void) {
	fd_set rfds;
	struct timeval tv;
	struct CountData cnt;

	/* Open RT-FIFO */
	if ((fd3 = open("/dev/rtf3", O_RDONLY)) < 0) {
		fprintf(stderr, "Error opening /dev/rtf3\n");
		exit(1);
	}

	FD_ZERO(&rfds);
	FD_SET(fd3, &rfds);
	tv.tv_sec = 0;
	/* 100ms */
	tv.tv_usec = 100000;
	
	/* Waiting For Response */
	if(select(FD_SETSIZE, &rfds, NULL, NULL, &tv) > 0){
		while(1){
			read(fd3, &cnt, sizeof(cnt));
			/* Waiting For Response (???) */
			if(select(FD_SETSIZE, &rfds, NULL, NULL, &tv) == 0)
				break;
		}
	}
	/* Close Handle */
	close(fd3);
	
	return cnt;
}
