/* ------------------------------------------------------------------------
 MIRS0202 PWM-Encoder Module Control Methods Header

                     From ATLMIRS Documents
                     Written by Hajime Wakabayashi
                     ver 0.1alpha Jan. 31 2004
   ------------------------------------------------------------------------ */
#ifndef _PWM_ENC_CTRL_H
#define _PWM_ENC_CTRL_H

/* For the task control */
#define COMMAND_FIFO  5
#define START_TASK    1   
#define STOP_TASK     2  
#define RESTART_TASK  3  
#define TUNE_SET     4

/* For PWM-Set */
#define MODE_KEEP	-999
#define MODE_DUTY	123
#define MODE_STOP	456
#define MODE_RETURN	789
#define MODE_LR		147
#define MODE_STR	148
#define MODE_HANKEI	149

/* For PWM-Get */
#define MODE_COUNT	200
#define MODE_SUM	201
#define MODE_DUMMY	202

/* Define the I/O ports */
#define PWM_LEFT_WRITE   0x0110
#define PWM_RIGHT_WRITE  0x0112
#define LEFT_COUNTER     0x0120
#define RIGHT_COUNTER    0x0122

struct Message{
	int Command;     /* Command to control RT-task */
	int Task;        /* number of the RT-task */
	int Period;      /* execution period of the RT-task */
};
struct CountData{
	int LCount;        /* pulses of left counter */
	int RCount;       /* pulses of right counter */
	int LSum;    /* sum of puslses of left counter */
	int RSum;    /* sum of puslses of right counter */
	int br;
};
struct SpeedData{
	int LSpeed;     /* velocity of left wheel */
	int RSpeed;     /* velocity of right wheel */
	int Tup;
	int Mode;
};

class PWM {
private:
	int fd3,fd4,fd5;

	void FSet(int,int,int,int);
	void FSetStatus(int, ...);
public:
	int cRb,cLb;

	PWM(void);
	~PWM(void);
	
	void Start(int);
	void Stop(void);
	void Restart(void);
	void Tune(int,int);

	void Set(int, ...);
	struct CountData Get();
};

#endif
