//----------------------------------------------------------------------------
// MIRS0202 sensor-method object
//                based on Eri.Taguchi
//                         Akihisa.Onoda
//                mod.  by Hajime Wakabayashi
//
//                     ver 0.1alpha Dec. 18 2003
//                         0.2      Jan. 30 2004

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <unistd.h>

#include "sensor.h"

int setcode=16;

// Construction --------------------------------------------------------------
Sensor::Sensor(void) {
}

// Destruction ---------------------------------------------------------------
Sensor::~Sensor(void) {
//	if (fd != NULL) close(fd);
//	if (fds != NULL) close(fds);
}

//----------------------------------------------------------------------------
int Sensor::TS(int SensorNo) {
	dOpen(TSWLS);

	if(SensorNo >=6) {
		fprintf(stderr, "Illegal sensor number.\n");
		dClose(TSWLS);
		return -1;
	} else {
		// read device
		ioctl(fd,TS_READ,data);
		// return decimal
		dClose(TSWLS);
		return (data[SensorNo] - 48);
	}
	dClose(TSWLS);
	return -1;
}

//----------------------------------------------------------------------------
int Sensor::WLS(int SensorNo) {
	dOpen(TSWLS);
	if(SensorNo >=5) {
		fprintf(stderr, "Illegal sensor number.\n");
		dClose(TSWLS);
		return -1;
	} else {
		// read device
		ioctl(fd,IRS_READ,data);
		// return decimal
		dClose(TSWLS);
		return (data[SensorNo] - 48);
	}
	dClose(TSWLS);
	return -1;
}

//----------------------------------------------------------------------------
int Sensor::USS(int SensorNo) {
	dOpen(SERIAL);
	switch(SensorNo) {
		case 0:
			send_data = 0x41; break;
		case 1:
			send_data = 0x42; break;
		case 2:
			send_data = 0x44; break;
		default:
			send_data = 0x41; break;
	}
	//send data
	if(write(fds,&send_data,1)<0){
		fprintf(stderr, "Cannot send data to RS232c\n");
		return -1;
	}
	//receive data
	read(fds, pic_data, 3);

	//cast
	temp[0] = (int)pic_data[0];
	temp[1] = (int)pic_data[1];

	//check header bit
	temp[2] = temp[0] & 0x80;
	temp[3] = temp[1] & 0x80;
	if(temp[2] == 0x80 && temp[3] == 0x80) {
		temp[4] = temp[0] & 0x7c;
		if((send_data == 0x41 && temp[4] == 0x40) ||
		   (send_data == 0x42 && temp[4] == 0x20) ||
		   (send_data == 0x44 && temp[4] == 0x10)) {
			temp[4] = temp[1] & 0x7f;
			temp[5] = (temp[0] & 0x03) << 7;

			dClose(SERIAL);
			return (temp[4] + temp[5]);
		} else {
			dClose(SERIAL);
			return -1;
		}
	} else {
		dClose(SERIAL);
		return -1;
	}
	dClose(SERIAL);
	return -1;
}
//----------------------------------------------------------------------------
int Sensor::IRS(int SensorNo) {
	dOpen(SERIAL);
	switch(SensorNo) {
		case 0:
			send_data = 0x48; break;
		case 1:
			send_data = 0x50; break;
		default:
			send_data = 0x48; break;
	}

	//send data
	if(write(fds,&send_data,1)<0){
		fprintf(stderr, "Cannot send data to RS232c\n");
		return -1;
	}
	//receive data
	read(fds, pic_data, 3);

	//cast
	temp[0] = (int)pic_data[0];
	temp[1] = (int)pic_data[1];

	//check header bit
	temp[2] = temp[0] & 0x80;
	temp[3] = temp[1] & 0x80;
	if(temp[2] == 0x80 && temp[3] == 0x80){
		temp[4] = temp[0] & 0x7c;
		if((send_data == 0x48 && temp[4] == 0x08) ||
		   (send_data == 0x50 && temp[4] == 0x04)) {
			dClose(SERIAL);
			return (temp[1] & 0x07);
		} else {
			dClose(SERIAL);
			return -1;
		}
	} else {
		dClose(SERIAL);
		return -1;
	}
	dClose(SERIAL);
	return -1;
}

//----------------------------------------------------------------------------
int Sensor::TS(void) {
	return (TS(5)*100000 + 
			TS(4)*10000 +
			TS(3)*1000 +
			TS(2)*100 +
			TS(1)*10 +
			TS(0) );
}

//----------------------------------------------------------------------------
int Sensor::WLS(void) {
	return (WLS(5)*100000 + 
			WLS(4)*10000 +
			WLS(3)*1000 +
			WLS(2)*100 +
			WLS(1)*10 +
			WLS(0) );
}

//----------------------------------------------------------------------------
void Sensor::dOpen(int type) {
	switch(type) {
		case TSWLS:
			//open device
			if((fd=open("/dev/irs_ts",O_RDWR)) < 0){
				fprintf(stderr, "Cannot open TS-WLS driver.\n");
				exit(EXIT_FAILURE);
			}
			break;
		case SERIAL:
			if((fds = open("/dev/ttyS0", O_RDWR)) < 0){
				fprintf(stderr, "Cannot open RS232c.\n");
				exit(EXIT_FAILURE);
			}
			break;
		default:
			break;
	}
}

//----------------------------------------------------------------------------
void Sensor::dClose(int type) {
	switch(type) {
		case TSWLS:
			close(fd);
			break;
		case SERIAL:
			close(fds);
			break;
		default:
			break;
	}
}
